import java.io.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

public class Main {

    private static ArrayList<Node> nodes = new ArrayList<>();
    private static ArrayList<Edge> edges = new ArrayList<>();

    private static Map<Integer,Integer> tmpMapping = new HashMap<>();

    private static boolean printMicroBenchOnlyNodes = false;
    private static String nodePrefix = "github.com";
    private static String[]  excludeStrings = {};

    public static void main(String args[]) {

        File systembenchfolder = new File("resources/system");

        for (File f : systembenchfolder.listFiles()) {
            System.out.println("Reading " + f.getAbsolutePath() + " ... ");
            readSystemBenchFile(f);
        }

        File microbenchfolder = new File("resources/micro");
        for (File f : microbenchfolder.listFiles()) {
            System.out.println("Reading " + f.getAbsolutePath() + " ... ");
            readMicroBenchFile(f);
        }

        writeCombinedGraphFile();

        int allNodes = 0;
        int redNodes = 0;
        int greenNodes = 0;
        for (Node n : nodes) {
            allNodes++;
            if (n.getColor().equals("#ff0000")) {
                redNodes++;
            }
            if (n.getColor().equals("#00ff00")) {
                greenNodes++;
            }
        }
        System.out.println("" + redNodes + "/" + allNodes + " nodes are red");
        System.out.println("" + greenNodes + "/" + allNodes + " nodes are green");
        System.out.println("" + (allNodes-redNodes-greenNodes) + "/" + allNodes + " nodes are black");

    }

    private static void writeCombinedGraphFile() {
        try {
            BufferedWriter bw = new BufferedWriter(new FileWriter(new File("combined.dot")));
            //Write header
            bw.write("digraph \"influxd\" {\n");
            bw.write("node [style=filled fillcolor=\"#f8f8f8\"]\n");
            bw.write("subgraph cluster_L { \"File: influxd\" [shape=box fontsize=16 label=\"Some Test: Pudding \" tooltip=\"influxd\"] }\n");
            //Write Nodes
            for (Node n : nodes) {
                bw.write("N" + n.getID() + "[label=\"" + n.getName() + "\" id=\"node" + n.getID() + "\" fontsize=10 shape=box tooltip=\"" + n.getTooltip() + "\" color=\"" + n.getColor() + "\" fillcolor=\"#ffffff\"]\n" );
            }

            //Write Edges
            for (Edge e : edges) {
                bw.write("N" + e.getStartNodeID() + " -> N" + e.getEndNodeID() + " [label=\"\" weight=" + e.getWeight() + " color=\"" + e.getColor() + "\" tooltip=\"" + e.getLabel() + "\" labeltooltip=\"\"]\n");
            }

            //Write end
            bw.write("}");

            bw.close();

        } catch (IOException e) {
            System.out.println("IOException: " + e.getMessage());
            e.printStackTrace();
        }
    }

    private static void readSystemBenchFile(File file) {
        tmpMapping.clear();
        try {
            BufferedReader br = new BufferedReader(new FileReader(file));
            String line = "";
            while ((line = br.readLine()) != null) {
                parseSystemBenchLine(line);
            }
        } catch (FileNotFoundException e) {
            System.out.println("File not found: " + file.getAbsolutePath());
            e.printStackTrace();
        } catch (IOException e) {
            System.out.println("IOException: " + e.getMessage());
            e.printStackTrace();
        }
    }

    private static void readMicroBenchFile(File file) {
        tmpMapping.clear();
        try {
            BufferedReader br = new BufferedReader(new FileReader(file));
            String line = "";
            while ((line = br.readLine()) != null) {
                parseMicroBenchLine(line, file.getName());
            }
        } catch (FileNotFoundException e) {
            System.out.println("File not found: " + file.getAbsolutePath());
            e.printStackTrace();
        } catch (IOException e) {
            System.out.println("IOException: " + e.getMessage());
            e.printStackTrace();
        }
    }

    private static void parseSystemBenchLine(String line) {
        if (line.startsWith("N")) {
            //System.out.println("Parsing " + line);
            //Node or edge
            if (line.contains("->")) {
                //Edge
                int startFileNode = Integer.parseInt(line.substring(1, line.indexOf(" ")));
                int start = line.indexOf("-> N") + 4;
                int end = line.indexOf(" ", start);
                int endFileNode = Integer.parseInt(line.substring(start, end));
                if (tmpMapping.containsKey(startFileNode) && tmpMapping.containsKey(endFileNode)) {
                    int startGraphNodeID = tmpMapping.get(startFileNode);
                    int endGraphNodeID = tmpMapping.get(endFileNode);

                    if (!doesEdgeExist(startGraphNodeID, endGraphNodeID)) {
                        Edge e = new Edge();
                        e.setLabel("Systemsbenchmark");
                        e.setStartNodeID(startGraphNodeID);
                        e.setEndNodeID(endGraphNodeID);
                        edges.add(e);
                    } else {
                        int weight = getEdge(startGraphNodeID, endGraphNodeID).getWeight() + 1;
                        getEdge(startGraphNodeID, endGraphNodeID).setWeight(weight);
                    }
                }
            } else {
                //Node
                int fileID = Integer.parseInt(line.substring(1,line.indexOf(" ")));

                int nameIndex = line.indexOf("tooltip=\"") + 9;
                String nodeName = line.substring(nameIndex, line.indexOf(" (", nameIndex));
                if (!doesNodeExist(nodeName)) {
                    Node n = new Node();
                    n.setName(nodeName);
                    n.setFromSystemBench(true);
                    n.setTooltip("Systemsbenchmark");
                    if (nodeName.startsWith(nodePrefix)) {
                        boolean exclude = false;
                        for (String ex : excludeStrings) {
                            if (nodeName.contains(ex)) {
                                exclude = true;
                            }
                        }
                        if (!exclude) {
                            nodes.add(n);
                            tmpMapping.put(fileID, n.getID());
                        }
                    }
                } else {
                    int weight = getNode(nodeName).getWeight() + 1;
                    getNode(nodeName).setWeight(weight);
                    tmpMapping.put(fileID, getNode(nodeName).getID());
                }
            }
        }
    }

    private static void parseMicroBenchLine(String line, String benchmarkName) {
        if (line.startsWith("N")) {
            //System.out.println("Parsing " + line);
            //Node or edge
            if (line.contains("->")) {
                //Edge
                int startFileNode = Integer.parseInt(line.substring(1, line.indexOf(" ")));
                int start = line.indexOf("-> N") + 4;
                int end = line.indexOf(" ", start);
                int endFileNode = Integer.parseInt(line.substring(start, end));
                if (tmpMapping.containsKey(startFileNode) && tmpMapping.containsKey(endFileNode)) {
                    int startGraphNodeID = tmpMapping.get(startFileNode);
                    int endGraphNodeID = tmpMapping.get(endFileNode);

                    if (!doesEdgeExist(startGraphNodeID, endGraphNodeID)) {
                        Edge e = new Edge();
                        e.setLabel(benchmarkName);
                        e.setStartNodeID(startGraphNodeID);
                        e.setEndNodeID(endGraphNodeID);
                        e.setColor("#00ff00");
                        if (printMicroBenchOnlyNodes) {
                            edges.add(e);
                        }
                    } else {
                        int weight = getEdge(startGraphNodeID, endGraphNodeID).getWeight() + 1;
                        String oldLabel = getEdge(startGraphNodeID, endGraphNodeID).getLabel();
                        getEdge(startGraphNodeID, endGraphNodeID).setWeight(weight);
                        if (oldLabel.startsWith("Systemsbenchmark")) {
                            getEdge(startGraphNodeID, endGraphNodeID).setColor("#ff0000");
                        }
                        String newLabel = oldLabel + " AND " + benchmarkName;
                        getEdge(startGraphNodeID, endGraphNodeID).setLabel(newLabel);
                    }
                }
            } else {
                //Node
                int fileID = Integer.parseInt(line.substring(1,line.indexOf(" ")));

                int nameIndex = line.indexOf("tooltip=\"") + 9;
                //System.out.println("NameIndex=" + nameIndex);
                String nodeName = line.substring(nameIndex, line.indexOf(" (", nameIndex));
                //System.out.println("NodeName= " + nodeName);
                if (!doesNodeExist(nodeName)) {
                    Node n = new Node();
                    n.setName(nodeName);
                    n.setColor("#00ff00");
                    n.setTooltip(benchmarkName);
                    if (nodeName.startsWith(nodePrefix) && printMicroBenchOnlyNodes) {
                        boolean exclude = false;
                        for (String ex : excludeStrings) {
                            if (nodeName.contains(ex)) {
                                exclude = true;
                            }
                        }
                        if (!exclude) {
                            nodes.add(n);
                            tmpMapping.put(fileID, n.getID());
                        }
                    }
                } else {
                    int weight = getNode(nodeName).getWeight() + 1;
                    getNode(nodeName).setWeight(weight);
                    if (getNode(nodeName).isFromSystemBench()) {
                        getNode(nodeName).setColor("#ff0000");
                    }
                    String oldLabel = getNode(nodeName).getTooltip();
                    String newLabel =  oldLabel + " AND " + benchmarkName;
                    getNode(nodeName).setTooltip(newLabel);
                    tmpMapping.put(fileID, getNode(nodeName).getID());
                }
            }
        }
    }

    private static boolean doesNodeExist(String nodeName) {
        for (Node n : nodes) {
            if (n.getName().equals(nodeName)) {
                return true;
            }
        }
        return false;
    }

    private static Node getNode(String nodeName) {
        for (Node n : nodes) {
            if (n.getName().equals(nodeName)) {
                return n;
            }
        }
        return null;
    }

    private static boolean doesEdgeExist(int startGraphID, int endGraphID) {
        for (Edge e : edges) {
            if (e.getStartNodeID() == startGraphID && e.getEndNodeID() == endGraphID) {
                return true;
            }
        }
        return false;
    }

    private static Edge getEdge(int startGraphID, int endGraphID) {
        for (Edge e : edges) {
            if (e.getStartNodeID() == startGraphID && e.getEndNodeID() == endGraphID) {
                return e;
            }
        }
        return null;
    }
}
