import java.io.*;
import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Set;

public class OverlapMain {

    //public static String prefix = "github.com/VictoriaMetrics/VictoriaMetrics";
    public static String prefix = "github.com/influxdata";

    public static void main(String[] args) {
        File systembenchfolder = new File("resources/overlapAnalysis/influx");
        HashMap<String, ArrayList<String>> functions = new HashMap<>();

        for (File f : systembenchfolder.listFiles()) {
            System.out.println("Reading " + f.getAbsolutePath() + " ... ");
            ArrayList<String> names = readFunctionNames(f);
            System.out.println("Found " + names.size() + " functions");
            functions.put(f.getName(), names);
            System.out.println("parsed " + f.getName());
        }

        System.out.println("removing other nodes (& keeping project-only nodes) ...");

        for (ArrayList<String> funcs : functions.values()) {
            System.out.println("before: " + funcs.size());
            for (int i = 0; i < funcs.size(); i++) {
                if (!funcs.get(i).startsWith(prefix)) {
                    funcs.remove(i);
                    i--;
                }
            }
            System.out.println("after: " + funcs.size());
        }

        System.out.println("------------------------");
        System.out.println("Project-only nodes:");
        for (String scenario : functions.keySet()) {
            System.out.println(scenario + " : " + functions.get(scenario).size());
        }
        System.out.println("Unique nodes:");
        for (String scenario : functions.keySet()) {
            System.out.println("Looking for unique nodes for scenario " + scenario + "...");
            ArrayList<String> uniques = new ArrayList<>();
            uniques.addAll(functions.get(scenario));
            System.out.println("Starting with " + uniques.size() + " functions");
            for (String s : functions.keySet()) {
                if (!s.equals(scenario)) {
                    System.out.println("checking against " + s);
                    for (String f : functions.get(s)) {
                        for (int i = 0; i < uniques.size(); i++) {
                            if (uniques.get(i).equals(f)) {
                                uniques.remove(i);
                                i--;
                            }
                        }
                    }
                    System.out.println("uniques functions remaining: " + uniques.size());
                }
            }
        }
        System.out.println("--------------------");
        System.out.println("Common nodes of all...");
        for (String scenario : functions.keySet()) {
            System.out.println("Looking for common nodes for scenario " + scenario + "...");
            ArrayList<String> commons = new ArrayList<>();
            for (String fun : functions.get(scenario)) {
                int cnt = 0;
                for (String s : functions.keySet()) {
                    if (!s.equals(scenario)) {
                        for (String f : functions.get(s)) {
                            if (f.equals(fun)) {
                                cnt++;
                            }
                        }
                    }
                }
                if (cnt == 2) {
                    commons.add(fun);
                }
            }
            System.out.println("found " + commons.size() + " common functions for scenario " + scenario);
        }

        System.out.println("-----------");
        System.out.println("all nodes...");
        ArrayList<String> all = new ArrayList<>();
        for (ArrayList<String> list : functions.values()) {
            for (String s : list) {
                if (!all.contains(s)) {
                    all.add(s);
                }
            }
        }
        System.out.println("total: " + all.size() + "nodes");

    }


    private static ArrayList<String> readFunctionNames(File file) {
        ArrayList<String> functionNames = new ArrayList<>();
        try {
            BufferedReader br = new BufferedReader(new FileReader(file));
            String line = "";
            while ((line = br.readLine()) != null) {
                String name = getFunctionName(line);
                if (name != null) {
                    for (String tmp : functionNames) {
                        if (tmp.equals(name)) {
                            System.out.println("!!!!!!!!!!#############!!!!!!!!!!!!");
                            System.out.println("FUNCTION:::: #########" + name);
                        }
                    }
                    functionNames.add(name);
                }
            }
        } catch (FileNotFoundException e) {
            System.out.println("File not found: " + file.getAbsolutePath());
            e.printStackTrace();
        } catch (IOException e) {
            System.out.println("IOException: " + e.getMessage());
            e.printStackTrace();
        }
        return functionNames;
    }

    private static String getFunctionName(String line) {
        if (line.startsWith("N")) {
            //System.out.println("Parsing " + line);
            //Node or edge
            if (line.contains("->")) {
                //Edge
                //System.out.println("found some edge ... no nothing");
            } else {
                //Node
                int nameIndex = line.indexOf("tooltip=\"") + 9;
                String nodeName = line.substring(nameIndex, line.indexOf(" (", nameIndex));
                return nodeName;
            }
        }
        return null;
    }
}
